/***************************************************************************

  Word2CHM

  Copyright (c) 2010 sinosoft , written by yuans.
  http://www.sinoreport.net

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************/
using System;

namespace Word2CHM.CHMDom
{
	/// <summary>
	/// CHM DOM node list
	/// </summary>
    [Serializable()]
	public class CHMNodeList : System.Collections.CollectionBase 
	{
        internal CHMNode OwnerNode = null;

		/// <summary>
		/// get node specify index
		/// </summary>
		public CHMNode this[ int index ]
		{
			get
            {
                return ( CHMNode ) this.List[ index ] ;
            }
		}

        /// <summary>
        /// get the last node in the list
        /// </summary>
        public CHMNode LastNode
        {
            get
            {
                if (this.Count > 0)
                    return (CHMNode)this.List[this.Count - 1];
                else
                    return null;
            }
        }

		/// <summary>
		/// Add node
		/// </summary>
		/// <param name="node">New node</param>
		/// <returns>index of new node in the list</returns>
		public int Add( CHMNode node )
		{
            if (OwnerNode != null)
            {
                node.Parent = OwnerNode;
            }
			return this.List.Add( node );
		}
		/// <summary>
		/// Remove node
		/// </summary>
		/// <param name="node">node</param>
		public void Remove( CHMNode node )
		{
			this.List.Remove( node );
		}
		/// <summary>
		/// get index of specify node
		/// </summary>
		/// <param name="node">node</param>
		/// <returns>index</returns>
		public int IndexOf( CHMNode node )
		{
			return this.List.IndexOf( node );
		}
		/// <summary>
		/// Detect whether list contains specify node
		/// </summary>
		/// <param name="node">node</param>
		/// <returns>contains specify node</returns>
		public bool Contains( CHMNode node )
		{
			return this.List.Contains( node );
		}
		/// <summary>
		/// Insert node
		/// </summary>
		/// <param name="index">index</param>
		/// <param name="node">node</param>
		public void Insert( int index , CHMNode node )
		{
			this.List.Insert( index , node );
		}
		/// <summary>
		/// add some nodes
		/// </summary>
		/// <param name="list">node list</param>
		internal void AddRange( CHMNodeList list )
		{
			this.InnerList.AddRange( list );
		}
	}//public class CHMNodeList : System.Collections.CollectionBase 
}